'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/* eslint-disable no-console */
const path = require('path');

class ServerManager {

  constructor() {
    this.socketId = 0;
    this.sockets = {};
  }

  manage(server) {
    var _this = this;

    return _asyncToGenerator(function* () {
      if (_this.server) {
        yield _this.close();
      }

      _this.server = server;

      if (!_this.server) {
        _this.logger.error('Server bundle did not export a http listener');
        _this.logger.info('Please export a http listener using `export default` syntax');

        throw new Error('Server bundle did not export a http listener.');
      } else if (typeof _this.server.on !== 'function') {
        const message = 'Cannot attach connection handler to listener because it is missing an `on` method';
        _this.logger.error(message);

        throw new Error(message);
      }

      _this.server.unref();

      const on = _this.server ? _this.server.on.bind(_this.server) : function () {};

      // listen to all connections so we can destroy them on restart
      on('connection', function (socket) {
        socket.unref();

        const socketId = _this.socketId = _this.socketId + 1;
        _this.sockets[socketId.toString()] = socket;

        socket.on('close', function () {
          return delete _this.sockets[socketId.toString()];
        });
      });
    })();
  }

  setLogger(logger) {
    this.logger = logger;
  }

  close() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      if (_this2.server) {
        Object.keys(_this2.sockets).forEach(function (socketId) {
          const socket = _this2.sockets[socketId.toString()];
          socket.destroy();
        });
        _this2.sockets = {};
        const close = _this2.server ? _this2.server.close.bind(_this2.server) : function (cb) {
          return cb();
        };

        yield new Promise(function (resolve) {
          return close(resolve);
        });

        _this2.server = null;
      }
    })();
  }
}

const sharedServerManager = new ServerManager();

module.exports = class ServerListenerPlugin {

  constructor(env, logger, serverManager = sharedServerManager) {
    this.env = env;
    this.logger = logger;
    this.serverManager = serverManager;

    this.serverManager.setLogger(logger);

    process.on('SIGINT', () => {
      this.serverManager.close();
    });
  }

  apply(compiler) {
    var _this3 = this;

    compiler.plugin('done', (() => {
      var _ref = _asyncToGenerator(function* (stats) {
        const bundleName = _this3.logger.getName();

        if (stats.hasErrors()) {
          _this3.logger.error(`Bundle "${bundleName}" compiled with errors, keeping previous server instance running`);
          return;
        }

        // Clear out all files from this build
        Object.keys(require.cache).forEach(function (modulePath) {
          if (modulePath.indexOf(compiler.options.output.path) === 0) {
            delete require.cache[modulePath];
          }
        });

        const serverBuildPath = path.resolve(compiler.options.output.path, compiler.options.output.filename);

        yield _this3.serverManager.close();

        // start server
        try {
          // const shouldOpenBrowser = !this.runningServer;

          const server = require(serverBuildPath).default; // eslint-disable-line global-require, max-len

          yield _this3.serverManager.manage(server);

          const port = _this3.env.getConfiguration().env.SERVER_PORT;
          const url = `http://localhost:${port}`;

          _this3.logger.info(`\tServer is listening on ${url}`);
        } catch (e) {
          _this3.logger.error('Error during server bundle start/restart');
          _this3.logger.log(e);

          throw e;
        }
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })());
  }
};